/*

             **************************************************
             ****             VEGA - ArgPar V1.2           ****
             ****              Arguments parser            ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "formats.h"
#include "globstr.h"

#define  MAXOPT         MAXTRJATM
#define  MAXSUBOPT      3


/**** Local definitions ****/

typedef struct  {
  char          *Name;          /* Name of mesure                  */
  WORD          Args;           /* Number of arguments             */
  float         Extra;          /* Extra parameter for calculation */
  ULONG         Calcs;          /* Required calculations           */
} MESTABLE;


/**** Local prototypes ****/

static BOOL AtmMatch(ULONG *, ULONG, UWORD);



/**** Copy the atom number ****/

static BOOL AtmMatch(ULONG *Dest, ULONG Src, UWORD Idesc)
{
  BOOL          Ret = TRUE;

  if (*Dest) Ret = CatErr(MSG_ERR_ARGPAR_MATCH, Idesc + 1);
  else *Dest = Src;

  return Ret;
}


/**** Translate atom description into atom number ****/

BOOL AtmTrans(ATOMO *InizAtm, ULONG TotAtm, register TRJPAR *TrjPar)
{
  ATMDESC               Tmp;
  register ATOMO        *Atm;
  register ATMDESC      *Desc;
  UWORD                 j, k;

  BOOL                  Ret = TRUE;

  if (TrjPar -> Com <= MS_TORSION) {
    for(k = 0; (Ret) && (k < TrjPar -> Num); ++k) {
      Desc = &TrjPar -> Atm[k];
      if (!Desc -> Num) {
        for(Atm = InizAtm; (Ret) && (Atm); Atm = Atm -> Ptr) {
          if (Atm -> Name.L == Desc -> Name.L) {
            if (Desc -> ResName.L) {
              if (Atm -> ResName.L == Desc -> ResName.L) {
                if (Desc -> ResSeq.L) {
                  if (Atm -> ResSeq.L == Desc -> ResSeq.L)
                    Ret = AtmMatch(&Desc -> Num, Atm -> Num, k);
                } else Ret = AtmMatch(&Desc -> Num, Atm -> Num, k);
              }
            } else Ret = AtmMatch(&Desc -> Num, Atm -> Num, k);
          }
        } /* End of ATOMO for*/
        if (!Desc -> Num) Ret = CatErr(MSG_ERR_ARGPAR_INPFILE, k + 1);
      } else if (Desc -> Num > TotAtm) Ret = CatErr(MSG_ERR_ARGPAR_ATMNUM, k + 1);
      for(j = 0; (Ret) && (j < TrjPar -> Num); ++j)
        if ((j != k) && (Desc -> Num == TrjPar -> Atm[j].Num))
          Ret = CatErr(MSG_ERR_ARGPAR_DUPDESC, k + 1, j + 1);
    } /* End of PAR for */

    /**** Sort atom descriptors ****/

    if (Ret) {
      for(j = 0; j < TrjPar -> Num; ++j) {
        Desc = &TrjPar -> Atm[j];
        for(k = j; k < TrjPar -> Num; ++k) {
          if (Desc -> Num > TrjPar -> Atm[k].Num)
            Desc = &TrjPar -> Atm[k];
        } /* End of for loop */
        if (Desc -> Num != TrjPar -> Atm[j].Num) {
          memcpy(&Tmp, &TrjPar -> Atm[j], sizeof(ATMDESC));
          memcpy(&TrjPar -> Atm[j], Desc, sizeof(ATMDESC));
          memcpy(Desc, &Tmp, sizeof(ATMDESC));
        }
      } /* End of for loop */
    }
  }

  return Ret;
}


/**** Analysis command parser ****/

BOOL TrjParser(TRJPAR *TrjPar)
{
  ATMDESC       *Desc;
  char          *Arg, *PrecArg, *SubOpt[MAXSUBOPT];
  ULONG         Com;
  UWORD         j, TotSubOpt;
  WORD          k;

  BOOL          Ret        = TRUE;


  const MESTABLE Mesure[] = {{ "DISTANCE",  2,  0.0           , CALC_NONE },
                             { "ANGLE"   ,  3,  0.0           , CALC_NONE },
                             { "TORSION" ,  4,  0.0           , CALC_NONE },
                             { "PLANEANG",  6,  0.0           , CALC_NONE },
                             { "VLOGP"   ,  0, VLOGP_PROBE_RAD, CALC_FFIELD|CALC_VDW },
                             { "DIPOLE"  ,  0,  0.0           , CALC_CHARGE },
                             { "SURFACE" , -1, -1.0           , CALC_VDW },
                             { "SURFDIA" , -1, -1.0           , CALC_VDW },
                             { "VOLUME"  ,  0,  0.0           , CALC_VDW },
                             { "VOLDIA"  ,  0,  0.0           , CALC_VDW },
                             { NULL      ,  0,  0.0           , CALC_NONE }
                            };

  if (TrjPar -> Num >= 1) {
    for(Com = 0; (Mesure[Com].Name) &&
                 (strcasecmp(TrjPar -> Arg[0], Mesure[Com].Name)); ++Com);
    if (Mesure[Com].Name) {
      TrjPar -> Com   = 1 << Com;
      TrjPar -> Calcs = Mesure[Com].Calcs;
      --TrjPar -> Num;
      k = Mesure[Com].Args;
      if ((k != (WORD)-1) ||
          ((k == (WORD)-1) && (TrjPar -> Num))) {
        if ((k == (WORD)-1) || (TrjPar -> Num == k)) {
          ++TrjPar -> Arg;
          for(k = 0; (Ret) && (k < TrjPar -> Num); ++k) {
            Desc              = &TrjPar -> Atm[k];
            Desc -> Num       = 0;
            Desc -> Name.L    = 0;
            Desc -> ResName.L = 0;
            Desc -> ResSeq.L  = 0;
            Arg               = TrjPar -> Arg[k];
            if (!SubArg(&Arg, SUBA_POSINT, FALSE)) {
              PrecArg   = Arg;
              SubOpt[0] = Arg;
              TotSubOpt = 1;

              /**** Parse the atom description ****/
              /**** Format: ATOM:RESN:RNUM     ****/

              while((Ret) && (*Arg)) {
                if (*Arg == ':') *Arg = 0;
                else if (!*PrecArg) {
                  if (TotSubOpt < MAXSUBOPT) SubOpt[TotSubOpt++] = Arg;
                  else Ret = CatErr(MSG_ERR_ARGPAR_INCDESC);
                }
                PrecArg = Arg++;
              } /* End of while */

              for(j = 0;(Ret) && (j < TotSubOpt); ++j) {
                if (strlen(SubOpt[j]) <= sizeof(QCHAR)) {
                  switch(j) {
                  case 0:         /* Atom name      */
                    ResConv(&TrjPar -> Atm[k].Name, SubOpt[j]);
                    break;
                  case 1:         /* Residue name   */
                    ResConv(&TrjPar -> Atm[k].ResName, SubOpt[j]);
                    break;
                  case 2:         /* Residue number */
                    ResConv(&TrjPar -> Atm[k].ResSeq, SubOpt[j]);
                    break;
                  }
                } else Ret = CatErr(MSG_ERR_ARGPAR_TOOLONG);
              } /* End of for loop */
            } else {
              sscanf(TrjPar -> Arg[k], "%d", &TrjPar -> Atm[k].Num);
              if (!TrjPar -> Atm[k].Num) Ret = CatErr(MSG_ERR_ARGPAR_ILLATMNUM);
            }
          } /* End of for loop */
        } else Ret = CatErr(MSG_ERR_ARGPAR_ILLNUMMES);
      }
    } else Ret = CatErr(MSG_ERR_ARGPAR_UNKMES);
  } else Ret = CatErr(MSG_ERR_ARGPAR_TOOFEW);


  return Ret;
}

