/*

             **************************************************
             ****          VEGA - Sybyl COMFA V1.0         ****
             ****                Field saver               ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************



*/

#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "formats.h"


/**** Local prototypes ****/

static BOOL    SybFldSave(FILE *, float *, ULONG);


/**** Calculate a COMFA field ****/

BOOL SybFldCalc(FILE *FH, ATOMO *InizAtm, ULONG TotAtm, SPACEREG *Reg)
{
  ATOMO         *Atm;
  float         *Fld, *InizFld, *VdwRad, *Vdw;
  float         Dist;
  ULONG         X, Y, Z;

  BOOL          Ret      = TRUE;
  float         *Contrib = NULL;

  if (Reg -> Par) {
    if (Reg -> Points) {
      if (VdwRad = AssignVdwRad(InizAtm, TotAtm, VLOGP_PROBE_RAD, NULL)) {
        switch (Reg -> Par -> Com) {
        case MS_VLOGP:
#ifdef VLOGP
          if (Contrib = (float *)Alloca(TotAtm * sizeof(float))) {
            FindRing(InizAtm);
            PrintProg(MSG_DCD_ASSATMTYPES);
            Ret = LogPCalc(InizAtm, TotAtm, (float *)&X, (float *)&Y, "BROTO", Contrib);
          } else Ret = 0;
#else
          Ret = PrintVlogPErr();
#endif
          break;
        } /* End of switch */

        if (Ret) {

          /**** Make the square of VDW radii ****/

          Vdw = VdwRad;
          for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
            *Vdw *= *Vdw;
            ++Vdw;
          }

          if (InizFld = (float *)Alloca(Reg -> Points * sizeof(float))) {
            Fld = InizFld;
            for(Z = 0; Z < Reg -> StZ; ++Z) {
              for(Y = 0; Y < Reg -> StY; ++Y) {
                for(X = 0; X < Reg -> StX; ++X) {
                  switch (Reg -> Par -> Com) {

#ifdef VLOGP
                  case MS_VLOGP:
                    *Fld = CalcSrfLogP(InizAtm, Contrib,
                                       X * Reg -> StepS.x + Reg -> Low.x,
                                       Y * Reg -> StepS.y + Reg -> Low.y,
                                       Z * Reg -> StepS.z + Reg -> Low.z,
                                       VdwRad);
                    break;
#endif
                  }
                  ++Fld;
                } /* End of X loop */
              } /* End of Y loop */
            } /* End of Z loop */

            Ret = SybFldSave(FH, InizFld, Reg -> Points);
          }
          FREE(InizFld);
        } else Ret = FALSE;

        /**** Free the resources ****/

        switch (Reg -> Par -> Com) {

#ifdef VLOGP
        case MS_VLOGP:
          if (Contrib) FREE(Contrib);
          break;
#endif

        }
        FREE(VdwRad);

      } else Ret = FALSE;
    } else Ret = CatErr(MSG_ERR_COMFA_RGN);
  } else Ret = CatErr(MSG_ERR_COMFA_FIELD);

  return Ret;

}


/**** Save a COMFA field in .fld format ****/

static BOOL SybFldSave(FILE *FH, register float *Fld, ULONG TotPts)
{
  BOOL                  Ret = TRUE;
  register ULONG        Pts;

  for(Pts = 1;(Ret) && (Pts <= TotPts); ++Pts) {
    if (fprintf(FH, "  %12.6f", *Fld++) > 0) {
      if (!(Pts % 5))
        if (fprintf(FH, "\n") <= 0) Ret = PrintDosErr();
    } else Ret = PrintDosErr();
  }

  if (TotPts % 5)
    if (fprintf(FH, "\n") <= 0) Ret = PrintDosErr();

  return Ret;
}


