/*

             **************************************************
             ****  VEGA - Gasteiger-Marsilli Charges V1.1  ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"

/**** Special atom types ****/

#ifdef LITTLE_ENDIAN
#define  GAST_C_CAT    0x00002b43	/* Type C+	*/
#define  GAST_N_CAT    0x002b334e	/* Type N3+	*/
#define  GAST_O_ANI    0x00002d4f	/* Type O-	*/
#else
#define  GAST_C_CAT    0x432b0000	/* Type C+	*/
#define  GAST_N_CAT    0x4e332b00	/* Type N3+	*/
#define  GAST_O_ANI    0x4f2d0000	/* Type O-	*/
#endif


typedef struct {
  float		a, b, c, d;
} GASTPAR;


BOOL Gasteiger(char *Template, ATOMO *InizAtm, ULONG TotAtm)
{
  char			Buf[256], *Ptr;
  FILE			*FH;
  float			*x, *xx;
  float			a, b, c, d;
  float			d1, d2, z1, q1;
  GASTPAR		*InizPar;
  QCHAR			AtmType;
  register ATOMO        *Atm, *Atm2;
  register GASTPAR	*Gast;
  UWORD			k;
  ULONG			i, j, t;

  BOOL			Chk   = TRUE;
  BOOL			Ret   = FALSE;
  ULONG			Ass   = 0;
  UWORD			cycle = 0;

  if (FH = (FILE *)fopen(Template, "r")) {
    if (InizPar = (GASTPAR *)Alloca(sizeof(GASTPAR) * TotAtm)) {
      if (xx = (float *)Alloca(sizeof(float) * TotAtm)) {

       /**** Load Gasteiger parameters ****/

        while((Chk) && (fgets(Buf, 255, FH))) {
          if ((*Buf != '\n') && (*Buf != '#') && (*Buf != ';')) {
            k         = 0;
            AtmType.L = 0;
            for(Ptr = Buf; *Ptr; ++Ptr)
              if ((*Ptr != '\n') && (*Ptr != ' ') && (*Ptr != '\t')) {
                AtmType.C[k++] = *Ptr;
                if (k == 4) break;
              } else if (k) break;
            if (*Ptr) {
              sscanf(Buf, "%*s %f %f %f", &a, &b, &c);
              d = a + b + c;
              Gast = InizPar;
              for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
                if (Atm -> Pot.L == AtmType.L) {
                  Gast -> a = a;
                  Gast -> b = b;
                  Gast -> c = c;
                  Gast -> d = d;
                  ++Ass;
                  if (Ass == TotAtm) {
                    Chk = FALSE;
                    break;
                  }
                }
                ++Gast;
              }
            }
          }
        }

        /**** Setting unknown atom types ****/

        Gast = InizPar;
        for(Atm = InizAtm; (Atm) && (Ass != TotAtm); Atm = Atm -> Ptr) {
          if ((Atm -> Pot.L == *(LONG *)"Du\0") || (Gast -> d == 0.0)) {
            Gast -> d = 1.0;
            ++Ass;
          }
          ++Gast;
        }

        /**** Setting formal charges ****/

        x    = xx;
        Gast = InizPar;
        for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
          switch(Atm -> Pot.L) {
          case GAST_C_CAT:	/* Type C+	*/
          case GAST_N_CAT:	/* Type N3+	*/
            Atm -> Charge = 1.0;
            break;
          case GAST_O_ANI:	/* Type O-	*/
            Atm -> Charge = -0.5;
            break;
          default:
            Atm -> Charge = 0;
          }
          *x = Gast -> a;
          ++Gast;
          ++x;
        }

        /**** Smoothing charges ****/

        z1 = 1;
        do {
          z1  *= 0.5;
          d1   = 0.0;
          i    = 0;
          for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
            if (InizPar[i].d != 1.0) {
              q1 = Atm -> Charge;
              for (j = 0; j < (ULONG)Atm -> NSost; j++) {
                t = (Atm -> Conn[j] -> Num) - 1;
                if (InizPar[t].d != 1.0) {
                  Atm2 = Atm -> Conn[j];
                  d2 = InizPar[t].d;
                  if (xx[t] > xx[i]) d2 = InizPar[i].d;
                  if (Atm -> Elem.S == *(WORD *)"H") d2 = 20.02;
                  if (Atm2 -> Elem.S == *(WORD *)"H") d2 = 20.02;
                  Atm -> Charge += (xx[t] - xx[i]) / d2 * z1;
                }
              }
              q1 = fabs(Atm -> Charge - q1);
              if (q1 > d1) d1 = q1;
            }
            ++i;
          }
          if (d1 >= 0.001) {
            Atm = InizAtm;
            for(i = 0; i < TotAtm; ++i) {
              xx[i] = InizPar[i].a + InizPar[i].b * Atm -> Charge +
                      InizPar[i].c * Quad(Atm -> Charge);
              Atm = Atm -> Ptr;
            }
          }
          cycle++;
        } while ((d1 > 0.001) && (cycle <= 5));
        Ret = TRUE;

        a = 0;
        for(Atm = InizAtm; Atm; Atm = Atm -> Ptr)
          a += Atm -> Charge;
        if (!GLOBSW_STDOUT) printf(GetStr(MSG_GASTEIGER_TOTCHG), a);
        FREE(xx);
      }
      FREE(InizPar);
    }
    fclose(FH);
  }

  return Ret;
}
