/*

             **************************************************
             ****            VEGA - IFF MOLE V1.2          ****
             ****               Loader & Saver             ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


Uses an Amiga IFF chunk structure to save molecule.
All chunks are optional !

*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "iff.h"

/**** Local prototypes ****/

static BOOL IffWrtHdr(FILE *, LONG, ULONG);


/**** Interchange File Format (IFF) Loader ****/

ATOMO *IFFLoad(FILE *IFFIN, ULONG IFFLen, ULONG *TotAtomi)
{
  ATOMO         *Atm;
  ATOMO         **ConMtx;
  BYTE          Size;
  IFFHDR        ChunkHdr;
  QCHAR         ResName;
  ULONG         nb, i, k, ResNum, Tot;
  UWORD         j;
  VECTYPE       DCoord;

  ATOMO          *InizAtm = (void *)-1;

  /**** Connectivity structure ****/

  struct __ConAt {
    ULONG i, j;
    UBYTE Type;
  } ConAt;

  /**** Switches ****/

  BOOL          Ret     = TRUE;
  BOOL          Tst1    = FALSE;  /* Single precision coordinates ? */
  BOOL          Tst2    = FALSE;  /* Residue number readed ?        */

  if (fread(&ChunkHdr, sizeof(IFFHDR), 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
    Swap(&ChunkHdr.Size);
    Swap(&ChunkHdr.ID);
#endif
    if (ChunkHdr.ID == IFF_ATOMO) {
      if (fread(&Tot, sizeof(ULONG), 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
          Swap(&Tot);
#endif
        if (Tot > 0) {

          /**** Allocate the memory for each atom ****/

          for(k = 0;(Ret) && (k < Tot);++k) {
            if ((Atm = AllocAtm(&InizAtm, TotAtomi)) != NULL) {
              if (fread(Atm -> Elem.C, 2, 1, IFFIN) == 1) {
                if (Atm -> Elem.C[1] == ' ') Atm -> Elem.C[1] = 0;
                *Atm -> Name.C    = *Atm -> Elem.C;
                Atm -> Name.C[1]  = Atm -> Elem.C[1];
              } else Ret = PrintDosErr();
            } else Ret = FALSE;
          } /* End of atom loop */
          if (Ret) {
            LastAtm      = Atm;
            Atm -> Flags = 2;
            if ((ConMtx = (ATOMO **)Alloca(sizeof(ATOMO *) * Tot)) != NULL) {
              for(Atm = InizAtm; Atm; Atm = Atm -> Ptr)
                ConMtx[Atm -> Num - 1] = Atm;
              while((Ret) && (fread(&ChunkHdr, sizeof(IFFHDR), 1, IFFIN))) {
#ifdef LITTLE_ENDIAN
                Swap(&ChunkHdr.Size);
                Swap(&ChunkHdr.ID);
#endif
                Atm = InizAtm;
                switch(ChunkHdr.ID) {

                case IFF_XYZ1:  /* Cartesian coordinate chunk (single precision) */

                  do {
                    if (fread(&Atm -> x, sizeof(float) * 3, 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
                      Swap(&Atm -> x);
                      Swap(&Atm -> y);
                      Swap(&Atm -> z);
#endif
                      Atm = Atm -> Ptr;
                    } else Ret = PrintDosErr();
                  } while((Ret) && (Atm));
                  Tst1 = TRUE;
                  break;

                case IFF_XYZ2:  /* Cartesian coordinate chunk (double precision) */

                 if (!Tst1) {
                   do {
                     if (fread(&DCoord, sizeof(VECTYPE), 1, IFFIN) == 1) {
#ifndef LITTLE_ENDIAN
                       Atm -> x = (float)DCoord.x;
                       Atm -> y = (float)DCoord.y;
                       Atm -> z = (float)DCoord.z;
#else
                       Atm -> x = (float)SwapD(DCoord.x);
                       Atm -> y = (float)SwapD(DCoord.y);
                       Atm -> z = (float)SwapD(DCoord.z);
#endif
                       Atm      = Atm -> Ptr;
                     } else Ret = PrintDosErr();
                   } while((Ret) && (Atm));
                 }
                 break;

                case IFF_CONX:  /* Connectivity chunk                            */

                  if (fread(&nb, sizeof(ULONG), 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
                    Swap(&nb);
#endif
                    for(k = 0;(Ret) && (k < nb);++k) {
                      if (fread(&ConAt, sizeof(ULONG) * 2 + 1, 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
                        Swap(&ConAt.i);
                        Swap(&ConAt.j);
#endif
                        --ConAt.i;
                        --ConAt.j;
                        for(j = 0;(j < MAXBOND) && (ConMtx[ConAt.i] -> Conn[j]);++j);
                        ConMtx[ConAt.i] -> Conn[j]  = ConMtx[ConAt.j];
                        ConMtx[ConAt.i] -> Order[j] = ConAt.Type;
                        for(j = 0;(j < MAXBOND) && (ConMtx[ConAt.j] -> Conn[j]);++j);
                        ConMtx[ConAt.j] -> Conn[j]  = ConMtx[ConAt.i];
                        ConMtx[ConAt.j] -> Order[j] = ConAt.Type;
                      } else Ret = PrintDosErr();
                    } /* End of for loop */
                    GlobSwitch[3] = 0;
                  } else Ret = PrintDosErr();
                  break;

                case IFF_IIUB: /* IUPAC-IUB chunk for atom names                 */

                  if (fread(&Size, sizeof(BYTE), 1, IFFIN) == 1) {
                    do {
                      if (fread(Atm -> Name.C, (int)Size, 1, IFFIN) != 1)
                        Ret = PrintDosErr();
                      Atm = Atm -> Ptr;
                    } while((Ret) && (Atm));
                  } else Ret = PrintDosErr();
                  break;

                case IFF_CALC: /* Atomic charge and force field chunks           */

                  k  = ChunkHdr.Size;
                  nb = ReadNull(IFFIN, FFName, 20);
                  while((k > nb) && (fread(&ChunkHdr, sizeof(IFFHDR), 1, IFFIN))) {
#ifdef LITTLE_ENDIAN
                    Swap(&ChunkHdr.ID);
                    Swap(&ChunkHdr.Size);
#endif

                    Atm  = InizAtm;
                    nb  += ChunkHdr.Size + 8;
                    switch(ChunkHdr.ID) {

                    case IFF_CHRG:  /* Charge chunk        */
                      do {
                        if (fread(&Atm -> Charge, sizeof(float), 1, IFFIN) != 1)
                          Ret = PrintDosErr();
#ifdef LITTLE_ENDIAN
                        else Swap(&Atm -> Charge);
#endif
                        Atm = Atm -> Ptr;
                      } while((Ret) && (Atm));
                      GlobSwitch[2] = TRUE;
                      break;

                    case IFF_ATYP:  /* Force field chunk   */
                      if (fread(&Size, 1, 1, IFFIN) == 1) {
                        do {
                          if (fread(&Atm -> Pot.C, (int)Size, 1, IFFIN) != 1)
                            Ret = PrintDosErr();
                          Atm = Atm -> Ptr;
                        } while((Ret) && (Atm));
                      } else Ret = PrintDosErr();
                      break;

                    default:       /* Don't read the chunk */
                      fseek(IFFIN, ChunkHdr.Size, SEEK_CUR);
                    }
                  } /* End of chunk loop */
                  GlobSwitch[2] = TRUE;
                  break;

                case IFF_RSNU:  /* Chunk of residue number                       */

                  do {
                    if (fread(Atm -> ResSeq.C, sizeof(QCHAR), 1, IFFIN) != 1)
                      Ret = PrintDosErr();
                    Atm = Atm -> Ptr;
                  } while((Ret) && (Atm));
                  Tst2 = TRUE;
                  break;

                case IFF_RSNA:  /* Residue name chunk                            */

                  if (fread(&j, sizeof(UWORD), 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
                    SwapW(&j);
#endif
                    for(k = 0;(Ret) && (k < j);++k) {
                      if ((fread(&ResNum, sizeof(QCHAR), 1, IFFIN) == 1) &&
                          (fread(ResName.C, sizeof(QCHAR), 1, IFFIN) == 1)) {
                        Size = 0;
                        for(Atm  = InizAtm; Atm; Atm = Atm -> Ptr) {
                          if ((Size) && (ResNum != (ULONG)Atm -> ResSeq.L)) break;
                          if (ResNum == (ULONG)Atm -> ResSeq.L) {
                            Atm -> ResName.L = ResName.L;
                            Size = 1;
                          }
                        } /* End of for loop */
                      } else Ret = PrintDosErr();
                    } /* End of for loop */
                  } else Ret = PrintDosErr();
                  break;

                case IFF_MOLN:  /* Name of molecule(s) chunk                     */

                  if (fread(&j, sizeof(UWORD), 1, IFFIN) == 1) {
#ifdef LITTLE_ENDIAN
                    SwapW(&j);
#endif
                    for(k = 0;(Ret) && (k < j);++k) {
                      if ((fread(&i , sizeof(ULONG), 1, IFFIN) == 1) &&
                          (fread(&nb, sizeof(ULONG), 1, IFFIN) == 1)) {
#ifdef LITTLE_ENDIAN
                        Swap(&i);
                        Swap(&nb);
#endif
                        while(getc(IFFIN));
                        ConMtx[i + nb - 2] -> Flags = 2;
                      } else Ret = PrintDosErr();
                    } /* End of for loop */
                  } else Ret = PrintDosErr();
                  break;

                default:       /* Generic chunk                                  */

                  fseek(IFFIN, ChunkHdr.Size, SEEK_CUR);
                }
              }
              FREE(ConMtx);
            } else Ret = FALSE;
          }

          if ((Ret) && (!Tst2)) {
            for(Atm = InizAtm; Atm; Atm = Atm -> Ptr)
              *Atm -> ResSeq.C = '1';
          }
        } else Ret = CatErr(MSG_ERR_IFF_INVFILE);
      } else Ret = PrintDosErr();
    } else Ret = CatErr(MSG_ERR_IFF_INVFILE);
  } else Ret = PrintDosErr();

  if ((!Ret) && (InizAtm != (void *)-1)) {
    PDBFree(InizAtm);
    InizAtm = (void *)-1;
  }

  return InizAtm;
}


/**** Interchange File Format (IFF) Saver ****/

BOOL IFFSave(FILE *OUT, ATOMO *AtmIniz, ULONG Tot)
{

  char              Name[2];
  IFFHDR            Hdr;
  register ATOMO    *Atm;
  ULONG             Count, Iniz;

#ifdef LITTLE_ENDIAN
  float             TmpF;
  ULONG             Tmp, Tmp2;
  UWORD             TmpW;
  XYZ               Coord;
#endif

  BOOL              Ret      = TRUE;
  UBYTE             SizeFF   = 0;
  UBYTE             SizeIIUB = 3;
  ULONG             nb       = 0;
  UWORD             NMol     = 0;
  UWORD             NumRes   = 1;

  Name[1] = ' ';


 /**** Calculates nb, NMol, NumRes, SizeFF and SizeIIUB ****/

  Iniz = AtmIniz -> ResSeq.L;
  for(Atm = AtmIniz; Atm; Atm = Atm -> Ptr) {
    if (Atm -> Flags > 1) ++NMol;
    if (Atm -> Name.C[3]) SizeIIUB = 4;
    if (Atm -> ResSeq.L != (LONG)Iniz) {
      Iniz = Atm -> ResSeq.L;
      ++NumRes;
    }
    for(Count = 0;(Atm -> Conn[Count]) && (Count < MAXBOND); ++Count)
      if (Atm -> Conn[Count] -> Num > Atm -> Num) ++nb;;
    for(Count = 0;(Count < 4) && (Atm -> Pot.C[Count]); ++Count);
    if (SizeFF < (UBYTE)Count) SizeFF = (UBYTE)Count;
  } /* End of atom loop */

  /**** Header chunk ****/

  Hdr.ID   = IFF_FORM;
  Hdr.Size = IFF_HDRLEN * (6 + *GlobSwitch + GlobSwitch[1] + GlobSwitch[2]) +
             4 + IFF_ATOMO_SIZE + IFF_XYZ1_SIZE + IFF_CONX_SIZE * GlobSwitch[1] +
             IFF_IIUB_SIZE * *GlobSwitch + IFF_MOLN_SIZE + IFF_COMM_SIZE +
             IFF_CALC_SIZE * GlobSwitch[2] + IFF_RSNU_SIZE + IFF_RSNA_SIZE;
#ifdef LITTLE_ENDIAN
  Swap(&Hdr.ID);
  Swap(&Hdr.Size);
#endif

  if ((fwrite(&Hdr, IFF_FORM_SIZE, 1, OUT) == 1) &&
      (fwrite("MOLE", 4, 1, OUT) == 1)) {

    /**** Atom names chunk ****/

    if (Ret = IffWrtHdr(OUT, IFF_ATOMO, IFF_ATOMO_SIZE)) {
#ifdef LITTLE_ENDIAN
      Tmp = SwapI(Tot);
      if (fwrite(&Tmp, sizeof(ULONG), 1, OUT) == 1) {
#else
      if (fwrite(&Tot, sizeof(ULONG), 1, OUT) == 1) {
#endif
        for(Atm = AtmIniz;(Ret) && (Atm); Atm = Atm -> Ptr) {
          if (!Atm -> Elem.C[1]) Atm -> Elem.C[1] = ' ';
          if (fwrite(Atm -> Elem.C, 2, 1,OUT) != 1)
            Ret = PrintDosErr();
        } /* End of atom loop */

        /**** Cartesian coordinates chunk ****/

        if (Ret) {
          if (Ret = IffWrtHdr(OUT, IFF_XYZ1, IFF_XYZ1_SIZE)) {
            for(Atm = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr) {
#ifdef LITTLE_ENDIAN
              Coord.x = SwapF(Atm -> x);
              Coord.y = SwapF(Atm -> y);
              Coord.z = SwapF(Atm -> z);
              if (fwrite(&Coord, sizeof(XYZ), 1, OUT) != 1)
#else
              if (fwrite(&Atm -> x, sizeof(float) * 3, 1, OUT) != 1)
#endif
                Ret = PrintDosErr();
            }

            /**** Connectivity chunk ****/

            if (Ret) {
              if (GlobSwitch[1]) {
                if (Ret = IffWrtHdr(OUT, IFF_CONX, IFF_CONX_SIZE)) {
#ifdef LITTLE_ENDIAN
                  Tmp = SwapI(nb);
                  if (fwrite(&Tmp, sizeof(ULONG), 1, OUT) == 1) {
#else
                  if (fwrite(&nb, sizeof(ULONG), 1, OUT) == 1) {
#endif
                    for(Atm = AtmIniz;(Ret) && (Atm); Atm = Atm -> Ptr) {
                      for(Count = 0;(Ret) && (Atm -> Conn[Count]) && (Count < MAXBOND); ++Count)
                        if (Atm -> Conn[Count] -> Num > Atm -> Num) {
#ifdef LITTLE_ENDIAN
                          Tmp  = SwapI(Atm -> Num);
                          Tmp2 = SwapI(Atm -> Conn[Count] -> Num);
                          if ((fwrite(&Tmp , sizeof(ULONG), 1, OUT) != 1) ||
                              (fwrite(&Tmp2, sizeof(ULONG), 1, OUT) != 1) ||
                              (fwrite(&Atm -> Order[Count], 1, 1, OUT) != 1))
#else
                          if ((fwrite(&Atm -> Num, sizeof(ULONG), 1, OUT) != 1) ||
                              (fwrite(&Atm -> Conn[Count] -> Num, sizeof(ULONG), 1, OUT) != 1) ||
                              (fwrite(&Atm -> Order[Count], 1, 1, OUT) != 1))
#endif
                            Ret = PrintDosErr();
                        }
                    } /* End of atom loop */
                  } else Ret = PrintDosErr();
                }
              }

              /**** IUPAC-IUB chunk ****/

              if (Ret) {
                if (*GlobSwitch) {
                  if (Ret = IffWrtHdr(OUT, IFF_IIUB, IFF_IIUB_SIZE)) {
                    if (fwrite(&SizeIIUB, 1, 1, OUT) == 1) {
                      for(Atm = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr)
                        if (fwrite(Atm -> Name.C, SizeIIUB, 1, OUT) != 1)
                          Ret = PrintDosErr();
                    } else Ret = PrintDosErr();
                  }
                }

                /**** Potential & charges chunk ****/

                if (Ret) {
                  if (GlobSwitch[2]) {
                    if (Ret = IffWrtHdr(OUT, IFF_CALC, IFF_CALC_SIZE)) {
                      if (fwrite(FFName, strlen(FFName) + 1, 1, OUT) == 1) {
                        if (Ret = IffWrtHdr(OUT, IFF_CHRG, IFF_CHRG_SIZE)) {
                          for(Atm  = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr) {
#ifdef LITTLE_ENDIAN
                            TmpF = SwapF(Atm -> Charge);
                            if (fwrite(&TmpF, sizeof(float), 1, OUT) != 1)
#else
                            if (fwrite(&Atm -> Charge, sizeof(float), 1, OUT) != 1)
#endif
                              Ret = PrintDosErr();
                          } /* End of atom loop */
                          if ((Ret) && (Ret = IffWrtHdr(OUT, IFF_ATYP, IFF_ATYP_SIZE))) {
                            if (fwrite(&SizeFF, 1, 1, OUT) == 1) {
                              for(Atm  = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr)
                                if (fwrite(Atm -> Pot.C, SizeFF, 1, OUT) != 1)
                                  Ret = PrintDosErr();
                            } else Ret = PrintDosErr();
                          }
                        }
                      } else Ret = PrintDosErr();
                    }
                  }

                  /**** Number of residues chunk ****/

                  if (Ret) {
                    if (Ret = IffWrtHdr(OUT, IFF_RSNU, IFF_RSNU_SIZE)) {
                      for(Atm  = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr)
                        if (fwrite(Atm -> ResSeq.C, sizeof(QCHAR), 1, OUT) != 1)
                          Ret = PrintDosErr();
                    }

                    /**** Name of residues chunk ****/

                    if (Ret) {
                      if (Ret = IffWrtHdr(OUT, IFF_RSNA, IFF_RSNA_SIZE)) {
#ifdef LITTLE_ENDIAN
                        TmpW = NumRes;
                        SwapW(&TmpW);
                        if (fwrite(&TmpW, sizeof(UWORD), 1, OUT) == 1) {
#else
                        if (fwrite(&NumRes, sizeof(UWORD), 1, OUT) == 1) {
#endif
                          Atm  = AtmIniz;
                          Iniz = Atm -> ResSeq.L;
                          if ((fwrite(&Atm -> ResSeq.C, sizeof(QCHAR), 1, OUT) == 1) &&
                              (fwrite(&Atm -> ResName.C, sizeof(QCHAR), 1, OUT) == 1)) {
                            do {
                              if (Atm -> ResSeq.L != Iniz) {
                                if ((fwrite(&Atm -> ResSeq.C, sizeof(QCHAR), 1, OUT) == 1) &&
                                    (fwrite(&Atm -> ResName.C, sizeof(QCHAR), 1, OUT) == 1)) {
                                  Iniz = Atm -> ResSeq.L;
                                  ++NumRes;
                                } else Ret = PrintDosErr();
                              }
                              Atm = Atm -> Ptr;
                            } while((Ret) && (Atm));
                          } else Ret = PrintDosErr();
                        } else Ret = PrintDosErr();
                      }

                      /**** Name of molecules chunk ****/

                      if (Ret) {
                        Count    = 1;
                        Iniz     = 1;
                        if (Ret = IffWrtHdr(OUT, IFF_MOLN, IFF_MOLN_SIZE)) {
#ifdef LITTLE_ENDIAN
                          TmpW = NMol;
                          SwapW(&TmpW);
                          if (fwrite(&TmpW, sizeof(UWORD), 1, OUT) == 1) {
#else
                          if (fwrite(&NMol, sizeof(UWORD), 1, OUT) == 1) {
#endif
                            NMol = 1;
                            for (Atm  = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr) {
                              if (Atm -> Flags > 1) {
#ifdef LITTLE_ENDIAN
                                Tmp  = SwapI(Iniz);
                                Tmp2 = SwapI(Count);
                                if ((fwrite(&Tmp , sizeof(ULONG), 1, OUT) == 1) &&
                                    (fwrite(&Tmp2, sizeof(ULONG), 1, OUT) == 1) &&
                                    (fprintf(OUT, "Molecule%4d%c", NMol, 0) >= 0)) {
#else
                                if ((fwrite(&Iniz, sizeof(ULONG), 1, OUT) == 1) &&
                                    (fwrite(&Count, sizeof(ULONG), 1, OUT) == 1) &&
                                    (fprintf(OUT, "Molecule%4d%c", NMol, 0) >= 0)) {
#endif
                                  Iniz = Count + 1;
                                  ++NMol;
                                } else Ret = PrintDosErr();
                              }
                              ++Count;
                            }
                          } else Ret = PrintDosErr();
                        }

                        /**** Comment chunk ****/

                        if ((Ret) && (Ret = IffWrtHdr(OUT, IFF_COMM, IFF_COMM_SIZE))) {
                          if (fwrite(VEGAHdr, IFF_COMM_SIZE, 1, OUT) != 1)
                            Ret = PrintDosErr();
                        }

                      }
                    }
                  }
                }
              }
            }
          }
        }
      } else Ret = PrintDosErr();
    }
  } else Ret = PrintDosErr();

  return Ret;
}


/**** Open an IFF file for read ****/
/**** and eventually check it   ****/

FILE *OpenIff(char *Name, ULONG Comp)
{
  FILE           *FH = NULL;
  IFFHDR         Hdr;

  if (FH = fopen(Name, "rb")) {
    if (ReadIffHdr(FH, &Hdr, IFF_FORM)) {
      if (fread(&Hdr, sizeof(ULONG), 1, FH) == 1) {

#ifdef LITTLE_ENDIAN
        Swap(&Hdr.ID);
#endif

        if (Comp != Hdr.ID) {
          CatErr(MSG_ERR_IFF_NOTIFF, (char *)&Hdr.ID);
          fclose(FH);
          FH = NULL;
        }
      } else {
        PrintDosErr();
        fclose(FH);
        FH = NULL;
      }
    }
  }

  return FH;
}

/**** Write the IFF Header ****/

static BOOL IffWrtHdr(FILE *FH, LONG ID, ULONG Size)
{
  BOOL            Ret = TRUE;

#ifdef LITTLE_ENDIAN
  Swap(&ID);
  Swap(&Size);
#endif

  if ((fwrite(&ID  , sizeof(LONG) , 1, FH) != 1) ||
      (fwrite(&Size, sizeof(ULONG), 1, FH) != 1))
    Ret = PrintDosErr();

  return Ret;
}

/**** Read an IFF header and eventually check it with ****/
/**** the Comp value                                  ****/

BOOL ReadIffHdr(FILE *FH, IFFHDR *Hdr, ULONG Comp)
{
  BOOL          Ret;

  /**** Strip the padding byte ****/

  if (!feof(FH)) {
    if (ftell(FH) & 1) fgetc(FH);
    if (fread(Hdr, sizeof(IFFHDR), 1, FH) == 1) {

#ifdef LITTLE_ENDIAN
      Swap(&Hdr -> ID);
      Swap(&Hdr -> Size);
#endif

      Ret = TRUE;
      if ((Comp != IFF_READ_NOCHK) && (Comp != IFF_READ_NODOSERR) &&
          (Comp != Hdr -> ID))
        Ret = CatErr(MSG_ERR_IFF_CORRUPTED);
    } else {
      if (Comp != IFF_READ_NODOSERR) PrintDosErr();
      Ret = FALSE;
    }
  } else Ret = FALSE;

  return Ret;
}


/**** Read to NULL character ****/

UWORD ReadNull(FILE *IN, register char *Str, UWORD Lim)
{
  register UWORD  Count;

  for(Count = 1;((*Str = getc(IN)) != (UBYTE)NULL) && (--Lim);++Count) ++Str;
  if (!Lim) while(getc(IN)) ++Count;

  return Count;
}

