/*

              **************************************************
              ****            VEGA - Loader V1.2            ****
              ****              Loader Module               ****
              **** Copyright 1996-2000, Alessandro Pedretti ****
              **************************************************



*/

#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "formats.h"
#include "globstr.h"
#include "iff.h"


/*************************************************/
/****         Autodetection subroutine        ****/
/****   return the pointer of atom structure  ****/
/**** or NULL if an error occurs or -2 if the ****/
/****         file contains data only         ****/
/*************************************************/

ATOMO *Loader(char *MolFile, ULONG *TotAtomi, ULONG NMol)
{
  char    FileName[512];
  char    *Str;
  FILE    *MOLIN;
  RECORD  Line;
  ULONG   IFFLen, Num;

  ATOMO   *BegPtr = NULL;
  BOOL    Brk     = FALSE;
  BOOL    Ret     = TRUE;
  ULONG   Count   = 0;

  *GlobSwitch     = 0;
  *TotAtomi       = 0;

  /**** Binary format recognition with automatic decompression ****/

  if ((MOLIN = PkOpen(MolFile, "rb", FALSE)) != NULL) {

    /**** Quanta/MSF ****/

    if (fread(Line.Line, sizeof(RECORD), 1, MOLIN) == 1) {;
      if (!strncmp(Line.Line + (sizeof(ULONG) * 4), "QUANTA", 6)) {
        PrintLoad(MSG_LOADER_MSF, NMol);
        BegPtr = MsfLoad(MOLIN, TotAtomi);
        GLOBSW_IFFIIUB = TRUE;
        GLOBSW_IFFCALC = TRUE;

        /**** DCD Quanta/CHARMm trajectory file ****/

      } else if (!strncmp(Line.Line + sizeof(ULONG), "CORD", 4)) {
        Prefs.DCD_FILE = MolFile;
        strcpy(FileName, MolFile);
        ChangeExt(FileName, "CRD");
        fclose(MOLIN);

        /**** Read the associated CRD file ****/

        if (GLOBSW_MESTRJ) {
          if ((MOLIN = fopen(FileName, "r")) != NULL) {
            if ((fgets(Line.Line, LINELEN, MOLIN)) &&
                (fgets(Line.Line, LINELEN, MOLIN))) {
              if (*Line.Line == '*') {
                PrintLoad(MSG_LOADER_CRD, NMol);
                BegPtr = CRDLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
              } else Ret = CatErr(MSG_ERR_LOADER_CRDUNK);
            } else Ret = PrintDosErr();
          } else Ret = CatErr(MSG_ERR_LOADER_CRDASSFILE);
        } else Ret = CatErr(MSG_ERR_LOADER_CRDMESURE);

      /**** IFF format ****/

      } else if (!strncmp(Line.Line, "FORM", 4)) {
        fseek(MOLIN, 4, SEEK_SET);
        if ((fread(&IFFLen  , sizeof(ULONG), 1, MOLIN) == 1) &&
            (fread(&Line.Hdr, sizeof(LONG) , 1, MOLIN) == 1)) {
#ifdef LITTLE_ENDIAN
          Swap(&IFFLen);
#endif
          if (!strncmp(Line.Line, "MOLE", 4)) {
            PrintLoad(MSG_LOADER_IFF, NMol);
            BegPtr = IFFLoad(MOLIN, IFFLen, TotAtomi);
            GLOBSW_IFFIIUB = TRUE;
          }
        } else Ret = PrintDosErr();
      } else {


        /**** ASCII format recognition ****/

        if ((MOLIN = PkReOpen(MOLIN, "r")) != NULL) {
          while((!Brk) && (fgets(Line.Line, LINELEN, MOLIN))) {

            switch(Count) {
            case 0:       /* 1st file line */

              /**** Sybyl rgn format ****/

              if (!strncmp(Line.Line, "Filename:  ", 11)) {
                PrintLoad(MSG_LOADER_SYBREG, NMol);
                if (SybRgnLoad(MOLIN, &Line, &Region))
                  BegPtr = (ATOMO *)-2;
                else BegPtr = (ATOMO *)-1;
                Brk = TRUE;
              }
            break;

            case 1:       /* 2nd file line */

              /**** CHARMm CRD format ****/

              if (*Line.Line == '*') {
                PrintLoad(MSG_LOADER_CRD, NMol);
                BegPtr         = CRDLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
                Brk            = TRUE;
              }
              break;

            case 2:       /* 3rd file line */

              /**** QMC format ****/

              if ((!strncmp(Line.Line,"   1 ",5)) &&
                 (Line.Line[15] == '.') && (Line.Line[26] == '.') &&
                 (Line.Line[37] == '.') && (Line.Line[73] == '.')) {
                PrintLoad(MSG_LOADER_QMC, NMol);
                BegPtr         = CSSRLoad(MOLIN, &Line, TotAtomi, CSSR_QMC);
                GLOBSW_IFFCALC = TRUE;
                Brk            = TRUE;
              }
              break;

            case 3:       /* 4th file line */

            /**** MoPac Internal format ****/

              if (!strncmp(Line.Line + 6,
                  "0.0000000  0      0.000000  0      0.000000  0    0    0    0", 61)) {
                fseek(MOLIN, 0, SEEK_SET);
                PrintLoad(MSG_LOADER_MOPINT, NMol);
                BegPtr         = MoPacArcLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
                Brk            = TRUE;
              }
              break;

            case 4:       /* 5th file line */

              /**** CSSR format ****/

              if ((!strncmp(Line.Line,"   1 ",5)) &&
                  (!strncmp(Line.Line + 9, "  ",2))) {
                PrintLoad(MSG_LOADER_CSSR, NMol);
                BegPtr         = CSSRLoad(MOLIN, &Line, TotAtomi, CSSR_PURE);
                GLOBSW_IFFCALC = TRUE;
                Brk            = TRUE;
              }
              break;

            } /* end of switch */

            if (!Brk) {

              /**** Biosym Archive 1 & 3 format ****/

              if (!strncmp("!BIOSYM archive", Line.Line, 15)) {
                PrintLoad(MSG_LOADER_CAR, NMol);
                BegPtr         = CarLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFIIUB = TRUE;
                GLOBSW_IFFCALC = TRUE;
                break;
              }

              /**** PDB format ****/

              if ((Line.Hdr == *(LONG *)PDBAtmRec[0]) ||
                  (Line.Hdr == *(LONG *)PDBAtmRec[1]) ||
                  (Line.Hdr == *(LONG *)"REMA")) {
                PrintLoad(MSG_LOADER_PDB, NMol);
                BegPtr         = PDBLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFIIUB = TRUE;
                break;
              }

              /**** MoPac .arc format ****/

              if (!strncmp(Line.Line, "          FINAL GEOMETRY OBTAINED", 33)) {
                PrintLoad(MSG_LOADER_MOPARC, NMol);
                BegPtr         = MoPacArcLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
                break;
              }

              /**** Sybyl Mol2 format ****/

              if (!strcmp(Line.Line, "@<TRIPOS>MOLECULE\n")) {
                PrintLoad(MSG_LOADER_MOL2, NMol);
                BegPtr         = Mol2Load(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
                break;
              }

              /**** Gromos format ****/

              if (!strcmp(Line.Line, "POSITION\n")) {
                PrintLoad(MSG_LOADER_GROMOS, NMol);
                BegPtr         = GromosLoad(MOLIN, &Line, TotAtomi);
                GLOBSW_IFFCALC = TRUE;
                break;
              }

              /**** XYZ format ****/

              if (*Line.Line != '\n') {
                Num = 1;
                for(Str = Line.Line; *Str; ++Str) {
                  if ((*Str >= 'A') && (*Str <= 'Z')) {
                    ++Str;
                    if (((*Str >= 'A') && (*Str <= 'Z')) || (*Str == ' ') ||
                        (*Str == '\t')) {
                      ++Str;
                      if (*Str == ' ') {
                        ++Str;
                        while(*Str) {
                          if ((*Str >= '0') && (*Str <= '9')) ++Num;
                          else if ((*Str != ' ') && (*Str != '\t') &&
                                   (*Str != '.') && (*Str != '+') &&
                                   (*Str != '-') && (*Str != '\n')) {
                            Num = 0;
                            break;
                          }
                          ++Str;
                        }
                        if (!Num) break;
                      }
                    }
                  }
                }
                if (Num >= 16) {
                  PrintLoad(MSG_LOADER_XYZ, NMol);
                  BegPtr = XyzLoad(MOLIN, &Line, TotAtomi);
                  break;
                }
              } /* End of XYZ format */
            }
            ++Count;
          } /* end of while loop */
        }
      }

      PkClose(MOLIN);
      if (BegPtr == (ATOMO *)-1) BegPtr = 0;
      else if ((!BegPtr) && (Ret)) CatErr(MSG_ERR_LOADER_UNKFORMAT);
    } else PrintDosErr();
  } else CatErr(MSG_ERR_LOADER_NOTFOUND);

  return BegPtr;
}



