/*

             **************************************************
             ****             VEGA - PDB V1.2              ****
             ****              Loader & saver              ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************




Supported PDB formats:

* PDB standard          read & write
* PDB non standard      read & write
* PDBQ                  read only
* PDBF (Fat subformat)  read & write

*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "formats.h"


/**** Universal PDB Loader ****/

ATOMO *PDBLoad(FILE *PDBIN, RECORD *PDBLin, ULONG *TotAtomi)
{
  ATOMO          *InizAtm;
  char           *j, ResSeq[20];
  LONG           k;
  register ATOMO *Atm;
  UWORD          l;

  ATOMO          *PrecAtm  = NULL;
  BOOL           FirstAtm  = FALSE;
  BOOL           IsPdbf    = FALSE;

  fseek(PDBIN, 0, SEEK_SET);
  while(fgets(PDBLin -> Line, LINELEN, PDBIN)) {
    if ((PDBLin -> Hdr == *(LONG *)"REMA") &&
        (!strncmp(PDBLin -> Line, "REMARK  77 EXTRA", 16))) {
      if (!IsPdbf) {
        PrintProg(MSG_PDB_FATFOUND);
        IsPdbf = TRUE;
      }
      if (Atm = AllocAtm(&InizAtm, TotAtomi)) {
      	sscanf(PDBLin -> Line + 26, "%s %f", Atm -> Pot.C, &Atm -> Charge);
      } else break;

    } else if ((PDBLin -> Hdr == *(LONG *)PDBAtmRec[0]) ||
               (PDBLin -> Hdr == *(LONG *)PDBAtmRec[1])) {
      if ((IsPdbf) && (!FirstAtm)) {
        Atm      = InizAtm;
        PrecAtm  = InizAtm;
        FirstAtm = TRUE;
      }
      if (!IsPdbf) Atm = AllocAtm(&InizAtm, TotAtomi);
      if (Atm) {
      	sscanf(PDBLin -> Line + 12, "%4s", Atm -> Name.C);
      	l = *Atm -> Name.C;
      	if (isdigit(l)) {
          for(k = 0;(k < 4) && (Atm -> Name.C[k]); ++k);
          for(j = Atm -> Name.C;j < (Atm -> Name.C + sizeof(Atm -> Name.C)); ++j)
            *j = j[1];
          Atm -> Name.C[--k] = l;
      	}
      	*Atm -> Elem.C = toupper(*Atm -> Name.C);
      	if (isalpha(PDBLin -> Line[12]))
      	  Atm -> Elem.C[1] = tolower(Atm -> Name.C[1]);
      	sscanf(PDBLin -> Line + 17, "%3s", Atm -> ResName.C);
      	Atm -> ChainID = PDBLin -> Line[21];
      	sscanf(PDBLin -> Line + 22, "%4s", ResSeq);
      	ResConv(&Atm -> ResSeq, ResSeq);
      	sscanf(PDBLin -> Line + 30, "%8f%8f%8f", &Atm -> x, &Atm -> y, &Atm -> z);
      	if (PDBLin -> Hdr == *(LONG *)PDBAtmRec[0]) Atm -> Flags = 0;
      	else Atm -> Flags = 1;
        PrecAtm = Atm;
        if (IsPdbf) Atm = Atm -> Ptr;
      } else break;
    }
    if (PDBLin -> Hdr == *(LONG *)"TER ") PrecAtm -> Flags += 2;
  }
  if (PrecAtm) PrecAtm -> Flags += 2;
  LastAtm = Atm;

  return InizAtm;
}


/**** PDB Saver ****/

BOOL PDBSave(FILE *OUT, ATOMO *AtmIniz, ULONG Tot, WORD Type)
{
  char           *j;
  register ATOMO *Atm;
  UWORD          l;
  WORD           Hdr;

  BOOL           Ret       = TRUE;
  float          Charge    = 0;
  ULONG          k         = 1;
  ULONG          NumConect = 0;
  ULONG          NumRem    = 3;
  UWORD          NumTer    = 0;

  if (fprintf(OUT, "REMARK   4\n" \
                   "REMARK   4 %s\n"\
                   "REMARK   4\n", VEGAHdr) >= 0) {

    if (Type == PDB_FAT) {
      for(Atm = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr) {
        if (fprintf(OUT, "REMARK  77 EXTRA %5d %-2.2s %-4.4s %8.4f\n",
                    Atm -> Num, Atm -> Elem.C, Atm -> Pot.C, Atm -> Charge) < 0)
          Ret = PrintDosErr();
      } /* End of atom loop */
      NumRem += Tot;
    }

    /**** Writes ATOMO/HETATM records ****/

    if (Ret) {
      for(Atm = AtmIniz; (Ret) && (Atm); Atm = Atm -> Ptr) {
        if (Type == PDB_NONSTD) Hdr = 0;
        else Hdr = bittst(0, Atm -> Flags);
        if (fprintf(OUT, "%-6s%5d ", PDBAtmRec[Hdr], k) >= 0) {
          l = Atm -> Name.C[3];
          if (Atm -> Elem.C[1]) {
            if (fprintf(OUT, "%-4.3s", Atm -> Name.C) < 0) {
              Ret = PrintDosErr();
              break;
            }
          } else if (isdigit(l)) {
            for(j = Atm -> Name.C + 3;j > Atm -> Name.C; --j)
              *j = *(j - 1);
            *Atm -> Name.C = l;
            if (fprintf(OUT, "%-4.4s", Atm -> Name.C) < 0) {
              Ret = PrintDosErr();
              break;
            }
          } else if (fprintf(OUT, " %-3.3s", Atm -> Name.C) < 0) {
            Ret = PrintDosErr();
            break;
          }

          if (Type == PDB_PDBQ) Charge = Atm -> Charge;
          if (fprintf(OUT," %-3.3s %c%4.4s    %8.3f%8.3f%8.3f  1.00 %5.2f\n",
                      Atm -> ResName.C, Atm -> ChainID, Atm -> ResSeq.C,
                      Atm -> x, Atm -> y, Atm -> z, Charge) >= 0) {
            if ((Type != PDB_NONSTD) && (Atm -> Flags > 1)) {
              ++k;
              if (fprintf(OUT, "TER   %5d      %-3.3s  %4.4s\n",
                          k, Atm -> ResName.C, Atm -> ResSeq.C) >= 0)
                ++NumTer;
              else Ret = PrintDosErr();
            }
            ++k;
          } else Ret = PrintDosErr();
        } else Ret = PrintDosErr();
      }  /* End of atom loop */

      if (Ret) {

        /**** Writes the connectivity ****/

        if (GLOBSW_CONNSAV) {
          for(Atm = AtmIniz;(Ret) && (Atm); Atm = Atm -> Ptr) {
            Hdr = 0;
            for(k = 0;(Ret) && (Atm -> Conn[k]) && (k < MAXBOND); ++k) {
              if (!Hdr) {
                if (fprintf(OUT, "CONECT%5d", Atm -> Num) >= 0) {
                  Hdr = 1;
                  ++NumConect;
                } else Ret = PrintDosErr();
              }
	          if ((Ret) && (fprintf(OUT,"%5d", Atm -> Conn[k] -> Num) < 0))
                Ret = PrintDosErr();
            } /* End of bond order loop */
            if ((Ret) && (Hdr) && (fprintf(OUT,"\n") < 0))
              Ret = PrintDosErr();
          } /* End of atom loop */
        }

        /**** Writes MASTER record ****/

        if ((Ret) && (Type != PDB_NONSTD) &&
            (fprintf(OUT,"MASTER %8d    0    0    0    0    0    0    0%5d%5d%5d    0\n",
                     NumRem, Tot, NumTer, NumConect) < 0))
          Ret = PrintDosErr();
        if ((Ret) && (fprintf(OUT, "END\n") < 0))
          Ret = PrintDosErr();
      }
    }
  } else PrintDosErr();

  return Ret;
}
