/*

             **************************************************
             ****             VEGA - Saver V1.2            ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#include <math.h>
#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "formats.h"


/**** Selector of saver routine ****/

BOOL Saver(char *FileName, FILEFORMAT *Form, ATOMO *Top, ULONG TotAtomi)
{
  ATOMO   *Atm;
  char    MopKeys[256];
  char    *OpenWrtOpt;
  FILE    *OUT;
  float   *Contrib;
  float   MopCharge, LogP, MomLip;

  BOOL    Ret          = TRUE;

  FFDESC  PepDesc[]    = { {"C", 3, 0, 0, 0},
                           {"O", 1, 0, 0, 1},
                           {"N", 9, 0, 0, 1}
                         };

  if (!(Form -> Id & FORM_TYPE_PLOT)) {
    if (*FileName) {
      if (!strchr(FileName, '.')) {
        strcat(FileName, ".");
        strcat(FileName, Form -> Ext);
      }

      /**** Select write mode ****/

      switch(Form -> Id) {
      case FORM_QUANTA_MSF:
        OpenWrtOpt = "rb+";
        break;
      case FORM_IFF:
      case FORM_QUANTA_LOGP:
      case FORM_QUANTA_MEP:
        OpenWrtOpt = "wb+";
        break;
      default:
        OpenWrtOpt = "w";
      }
      OUT = fopen(FileName, OpenWrtOpt);
    } else OUT = stdout;

    if (OUT) {
      if ((Form -> Id != FORM_QUANTA_SRF) && (Form -> Id != FORM_BIOSYM_SRF) &&
          (Form -> Id != FORM_QUANTA_LOGP) && (Form -> Id != FORM_QUANTA_MEP) &&
          (Form -> Id != FORM_VRML_SRF_POINTS) && (Form -> Id != FORM_ENER_CVFF))
        PrintProg(MSG_SAVER_SAVEPROG);
      switch(Form -> Id) {
      case FORM_COMFAFLD:                  /* COMFA field .fld */
        Ret = SybFldCalc(OUT, Top, TotAtomi, &Region);
        break;
      case FORM_PDB:                       /* PDB standard     */
        Ret = PDBSave(OUT, Top, TotAtomi, PDB_PURE);
        break;
      case FORM_PDB_NONSTD:                /* PDB non standard */
        Ret = PDBSave(OUT, Top, TotAtomi, PDB_NONSTD);
        break;
      case FORM_BIOSYM_OLD:                /* Old Biosym       */
        Ret = CarSave(OUT, Top, BIOSYM_OLD);
        break;
      case FORM_BIOSYM:                    /* New Biosym       */
        Ret = CarSave(OUT, Top, BIOSYM_NEW);
        break;
      case FORM_IFF:                       /* IFF MOLE File    */
        Ret = IFFSave(OUT, Top, TotAtomi);
        break;
      case FORM_CSSR:                      /* CSSR             */
        Ret = CSSRSave(OUT, Top, TotAtomi, CSSR_PURE);
        break;
      case FORM_MOPINT:                    /* MoPac Internal   */
        if (!Prefs.MOPAC_FLAG) {
          if (Prefs.MOPAC_CRG == (WORD)32000) {
            if (!ChkCharges(Top, TotAtomi, &MopCharge)) {
              PrintWarn(GetStr(MSG_SAVER_WARN_INCCHARGE));
              MopCharge = 0.0;
            }
          } else MopCharge = (float)Prefs.MOPAC_CRG;
  #ifdef WIN32
          sprintf(MopKeys, "%s CHARGE=%.0f ", Prefs.MOPAC_DEF,
                MopCharge);
  #else
          sprintf(MopKeys, "%s CHARGE=%d ", Prefs.MOPAC_DEF,
                (LONG)rint((double)MopCharge));
  #endif
          if (Prefs.MOPAC_MMOK == 2) {
            if (MatchMtx = (UBYTE *)Alloca(TotAtomi)) {
              for(Atm = Top; Atm; Atm = Atm -> Ptr) {
                if (ChkFFAtm(Atm, &PepDesc[0])) {
                  memset(MatchMtx, 0, TotAtomi);
                  if (MatchAtm(Atm, &PepDesc[1], &PepDesc[2])) {
                    strcat(MopKeys, "MMOK");
                    break;
                  }
                }
              }
              FREE(MatchMtx);
            }
          } else if (!Prefs.MOPAC_MMOK) strcat(MopKeys, "MMOK");
            else strcat(MopKeys, "NOMM");
        } else strcpy(MopKeys, Prefs.MOPAC_DEF);
        Ret = MoPacSave(OUT, Top, TotAtomi, FileName, MopKeys);
        break;
      case FORM_XYZ:                      /* XYZ              */
        Ret = XyzSave(OUT, Top);
        break;
      case FORM_PDB_Q:                    /* PDBQ             */
        Ret = PDBSave(OUT, Top, TotAtomi, PDB_PDBQ);
        break;
      case FORM_QUANTA_SRF:               /* Quanta Surface   */
        Ret = Surface(OUT, FileName, Top, TotAtomi, Prefs.SAS_POINTS,
                      Prefs.SAS_PROBERAD, SRF_QUANTA, NULL);
        break;
      case FORM_BIOSYM_SRF:               /* Biosym Surface   */
        Ret = Surface(OUT, FileName, Top, TotAtomi, Prefs.SAS_POINTS,
                      Prefs.SAS_PROBERAD, SRF_INSIGHT, NULL);
        break;
      case FORM_PDB_FAT:                  /* PDBF             */
        Ret = PDBSave(OUT, Top, TotAtomi, PDB_FAT);
        break;
      case FORM_QMC:                      /* QMC              */
        Ret = CSSRSave(OUT, Top, TotAtomi, CSSR_QMC);
        break;
      case FORM_QUANTA_MSF:               /* Quanta MSF       */
        Ret = MsfWrite(OUT, Top, TotAtomi);
        break;
      case FORM_VRML_SRF_POINTS:          /* VRML SRF         */
        Ret = Surface(OUT, FileName, Top, TotAtomi, Prefs.SAS_POINTS,
                      Prefs.SAS_PROBERAD, SRF_VRML_POINTS, NULL);
        break;
      case FORM_VRML_SRF_CPK:             /* VRML SRF CPK     */
        Ret = VrmlCpkSave(OUT, Top, TotAtomi);
        break;
      case FORM_VRML:                     /* VRML             */
        Ret = VrmlSave(OUT, Top, TotAtomi);
        break;
      case FORM_VRML_SOLID:               /* VRML Solid       */
        Ret = VrmlSolSave(OUT, Top, TotAtomi);
        break;
      case FORM_INFO:                     /* Info             */
        Ret = InfoSave(OUT, Top, TotAtomi);
        break;
      case FORM_FASTA:                    /* Fasta            */
        Ret = FastaSave(OUT, Top);
        break;
      case FORM_ENER_CVFF:                /* CVFF energy      */
        Ret = CalcEnerCVFF(OUT, Top, TotAtomi, Prefs.ENRESNAME.L, Prefs.ENRRESSEQ.L);
        break;
      case FORM_MOL2:                     /* Mol2             */
        Ret = Mol2Save(OUT, Top, TotAtomi, FileName);
        break;
      case FORM_CRD:                      /* CHARMm CRD       */
        Ret = CRDSave(OUT, Top, TotAtomi);
        break;
      case FORM_GROMOS:                   /* Gromos Amstrong  */
        Ret = GromosSave(OUT, Top, TotAtomi, 1.0);
        break;
      case FORM_GROMOS_NM:                 /* Gromos nm       */
        Ret = GromosSave(OUT, Top, TotAtomi, 10.0);
        break;
      case FORM_QUANTA_LOGP:              /* Quanta LogP      */
        if (Contrib = (float *)Alloca(TotAtomi * sizeof(float))) {
          PrintProg(MSG_SAVER_ATTYPROG);
          if (Ret = LogPCalc(Top, TotAtomi, &LogP, &MomLip, "BROTO", Contrib)) {
            printf(GetStr(MSG_SAVER_LOGPBROTO), LogP, MomLip);
            Ret = Surface(OUT, FileName, Top, TotAtomi, Prefs.SAS_POINTS,
                          1.4, SRF_QUANTA_LOGP, Contrib);
          }
          FREE(Contrib);
        } else Ret = FALSE;
        break;
      case FORM_QUANTA_MEP:              /* Quanta MEP        */
        Ret = Surface(OUT, FileName, Top, TotAtomi, Prefs.SAS_POINTS,
                      0.0 , SRF_QUANTA_MEP, NULL);
        break;
      }
      fclose(OUT);

      if (Ret) {
        if ((Form -> Flags & FORM_FLAG_CHKCHG) && (!ChkCharges(Top, TotAtomi, NULL)))
          PrintWarn(GetStr(MSG_SAVER_WARN_NOTCHARGE));
        if((Form -> Flags & FORM_FLAG_CHKFF) && (!ChkForceField(Top, TotAtomi)))
          PrintWarn(GetStr(MSG_SAVER_WARN_FFNOTASS));
      }
    } else Ret = PrintDosErr();
  } else Ret = CatErr(MSG_ERR_SAVER_PLOT);

  return Ret;
}
