/*

             **************************************************
             ****        VEGA - System Utilities V1.0      ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"

#ifndef WIN32
#include <unistd.h>
#include <errno.h>
#else
#include <stdlib.h>
#ifdef __BORLANDC__
extern char     *VEGA_FileName;
#endif
#endif

#ifdef AMIGA
#include <dos/dos.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/execbase.h>
#endif

extern int   errno;


/**** Allocate the memory with error check ****/

void *Alloca(ULONG Mem)
{
  void		*Ind;

  if ((Ind = (void *)ALLOC(Mem)) == NULL)
    CatErr(MSG_ERR_UTILS_OUTOFMEM);

  return Ind;
}


/**** Shows an error ****/

BOOL Errore(const char *Err, ...)
{
  char          Buf[128];
  va_list       vl;

#ifndef WIN32
  sprintf(Buf, "\n\033[33m%s\033[0m ", GetStr(MSG_ERR_LOCALE_ERROR));
#else
  sprintf(Buf, "\n%s: ", GetStr(MSG_ERR_LOCALE_ERROR));
#endif
  va_start(vl, Err);
  strcat(Buf, Err);
  strcat(Buf, ".\n\n");
  vfprintf(stderr, Buf, vl);
  va_end(vl);

  return FALSE;
}


/**** Calculate the file length ****/

ULONG FileLength(FILE *FH)
{
  ULONG         Cur, Len;

  Cur = ftell(FH);            /* Current position            */
  fseek(FH, 0, SEEK_END);
  Len = ftell(FH);            /* Length of file              */
  fseek(FH, Cur, SEEK_SET);   /* Return to starting position */

  return Len;
}


/**** Copy in a string the path of Vega   ****
 **** The VEGADIR environment variable    ****
 **** it's only needed for Unix systems,  ****
 **** but if setted can be used also from ****
 **** other operating systems             ****/

BOOL GetPrgPath(char *Str, BOOL DataDir)
{
  char    *CharPtr;

  if (!getenv("VEGADIR")) {
#ifdef AMIGA
    strcpy(Str, "PROGDIR:");
#else
#  ifdef __BORLANDC__
   strcpy(Str, VEGA_FileName);
   CharPtr = Str + strlen(Str) - 1;
   do {
     if (*CharPtr == '\\') {
       *CharPtr = 0;
       break;
     }
   } while(*--CharPtr);
#  else
    return CatErr(MSG_ERR_UTILS_VEGADIR);
#  endif
#endif
  } else strcpy(Str, (char *)getenv("VEGADIR"));

  CharPtr = Str + strlen(Str) - 1;

#ifdef WIN32
  if (*CharPtr != '\\') strcat(Str, "\\");
#else
  if ((*CharPtr != ':') && (*CharPtr != '/')) strcat(Str, "/");
#endif

  /**** If requested, adds the Data subdirectory ****/

  if (DataDir) strcat(Str, "Data" C_DIRSEP);

  return TRUE;
}


/**** Show a system error ****/

BOOL PrintDosErr(void)
{
#ifdef WIN32
  fprintf(stderr, "\n%s %s\n",
          GetStr(MSG_ERR_UTILS_DOSERR), strerror(errno));
#else
  fprintf(stderr, "\n\033[33m%s\033[0m %s.\n\n",
          GetStr(MSG_ERR_UTILS_DOSERR), strerror(errno));
#endif

  return FALSE;
}


/**** Print warning message ****/

void PrintWarn(char *Msg)
{
  if (!GLOBSW_STDOUT)
#ifdef WIN32
    printf("\n%s %s\n\n", GetStr(MSG_WARN_UTILS_WARN), Msg);
#else
    printf("\n\033[33m%s\033[0m %s\n\n", GetStr(MSG_WARN_UTILS_WARN), Msg);
#endif
}

/**** Change endian ****/

void Swap(register void *Val)
{
  register UBYTE    T;

  T = ((UBYTE *)Val)[0];
  ((UBYTE *)Val)[0] =  ((UBYTE *)Val)[3];
  ((UBYTE *)Val)[3] = T;
  T = ((UBYTE *)Val)[1];
  ((UBYTE *)Val)[1] =  ((UBYTE *)Val)[2];
  ((UBYTE *)Val)[2] = T;
}


/**** Change the endian for float ****/

double SwapD(double Num)
{
  register UBYTE    T;
  register UBYTE    *Val = (UBYTE *)&Num;

  T = ((UBYTE *)Val)[0];
  ((UBYTE *)Val)[0] =  ((UBYTE *)Val)[7];
  ((UBYTE *)Val)[7] = T;
  T = ((UBYTE *)Val)[1];
  ((UBYTE *)Val)[1] =  ((UBYTE *)Val)[6];
  ((UBYTE *)Val)[6] = T;
  T = ((UBYTE *)Val)[2];
  ((UBYTE *)Val)[2] =  ((UBYTE *)Val)[5];
  ((UBYTE *)Val)[5] = T;
  T = ((UBYTE *)Val)[3];
  ((UBYTE *)Val)[3] =  ((UBYTE *)Val)[4];
  ((UBYTE *)Val)[4] = T;

  return Num;
}


/**** Change the endian for float ****/

float SwapF(float Val)
{
  Swap(&Val);
  return Val;
}


/**** Change the endian for int ****/

int SwapI(int Val)
{
  Swap(&Val);
  return Val;
}


/**** Change the endian for word ****/

void SwapW(register void *Val)
{
  register UBYTE    T;

  T = ((UBYTE *)Val)[0];
  ((UBYTE *)Val)[0] =  ((UBYTE *)Val)[1];
  ((UBYTE *)Val)[1] = T;
}

