/*

             **************************************************
             ****             VEGA - Utils V1.3            ****
             ****          Utilities & Subroutines         ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"

#ifdef AMIGA
#include <dos/dos.h>
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/execbase.h>
#endif


static ULONG AtmLoaded = 0;


/**** Add a value to the radius matrix ****/

void AddToRad(register ATOMO *Atm, register float *Rad, register float Val)
{
  while(Atm) {
    *Rad += Val;
    Atm = Atm -> Ptr;
    ++Rad;
  }
}


/**** Allocates the description structure for one atom  ****/
/**** using the S.O. specific memory allocation routines ****/

ATOMO *AllocAtm(ATOMO **InizAtm, ULONG *TotAtomi)
{
  register ATOMO   *Atm;
  static ATOMO     *PrecAtm;

  if (Atm = (ATOMO *)SOALLOC(sizeof(ATOMO))) {
    if (!*TotAtomi) *InizAtm = Atm;
    else PrecAtm -> Ptr = Atm;
    ++*TotAtomi;
    PrecAtm           = Atm;
    *Atm  -> ResSeq.C = '1';
    Atm  -> ChainID   = ' ';
    Atm  -> Num       = ++AtmLoaded;
    *Atm -> Elem.C    = '?';
    *Atm -> Pot.C     = '?';
    Atm -> ResName.L = *(LONG *)"UNK\0";
  } else {
    if (*TotAtomi) {
      PDBFree(*InizAtm);
      *InizAtm = NULL;
    }
    CatErr(MSG_ERR_UTILS_OUTOFMEM);
  }

  return Atm;
}


/**** Change or add the file extension ****/

void ChangeExt(char *Str, char *Ext)
{
  register char *Ptr;

  for(Ptr = Str + strlen(Str) - 1; Ptr != Str; --Ptr)
    if (*Ptr == '.') {
      strcpy(Ptr + 1, Ext);
      break;
    }
  if ((Ptr == Str) && (*Ptr != '.')) {
    strcat(Ptr, ".");
    strcat(Ptr, Ext);
  }
}

/**** Check if the charges are assigned ****/

BOOL ChkCharges(register ATOMO *Atm, ULONG TotAtm, float *TotCharge)
{
  BOOL            Ret       = TRUE;
  register float  Chg       = 0;
  register ULONG  Uncharged = 0;

  do {
    if (Atm -> Charge == 0.0) ++Uncharged;
    Chg += Atm -> Charge;
    Atm = Atm -> Ptr;
  } while(Atm);
  if (Uncharged != TotAtm) {
    if (TotCharge) *TotCharge = Chg;
  } else Ret = FALSE;

  return Ret;
}


/**** Check if the force field is assigned ****/

BOOL ChkForceField(register ATOMO *Atm, ULONG TotAtm)
{
  register ULONG  Unassigned = 0;

  do {
    if (Atm -> Pot.L == 0x3f000000) ++Unassigned;
    Atm = Atm -> Ptr;
  } while(Atm);
  if (Unassigned == TotAtm) return FALSE;
  else return TRUE;
}


/**** Counts residues ****/

ULONG CountRes(register ATOMO *Atm)
{
  register ULONG  PrecResName = 0;
  register ULONG  PrecResSeq  = 0;
  register ULONG  TotRes      = 0;

  while(Atm) {
    if ((Atm -> ResName.L != (LONG)PrecResName) ||
        (Atm -> ResSeq.L  != (LONG)PrecResSeq)) {
      PrecResName = Atm -> ResName.L;
      PrecResSeq  = Atm -> ResSeq.L;
      ++TotRes;
    }
    Atm = Atm -> Ptr;
  }

  return TotRes;
}


/**** Find a residue ****/

ULONG FindResPtr(register ATOMO *Atm, ATOMO **ResPtr, LONG NomeRes, LONG NRes)
{
  register UWORD  Chk;
  register ULONG  NAtm = 0;

  *ResPtr = 0;

  while(Atm) {
    Chk = 0;
    if (!NomeRes) Chk = 1;
    else if (Atm -> ResName.L == NomeRes) Chk = 1;
    if (!NRes) ++Chk;
    else if (Atm -> ResSeq.L == NRes) ++Chk;
    if (Chk == 2) {
      if (!*ResPtr) *ResPtr = Atm;
      ++NAtm;
    }
    Atm = Atm -> Ptr;
  }

  return NAtm;
}


/**** Reads a record in Fortran format ****/

BOOL ForRead(FILE *FH, void *Ptr, ULONG Len)
{
  BOOL            Ret = TRUE;

  fseek(FH, sizeof(int), SEEK_CUR);
  if (fread(Ptr, Len, 1, FH) == 1)
    fseek(FH, sizeof(int), SEEK_CUR);
  else Ret = PrintDosErr();

  return Ret;
}


/**** Performs Seek from/to Fortran records ****/

BOOL ForSeek(FILE *FH, ULONG Rec)
{
  unsigned int  Len;
  BOOL    Ret = TRUE;

  do {
    if (fread(&Len, sizeof(int), 1, FH)) {
#ifdef LITTLE_ENDIAN
      Swap(&Len);
#endif
      fseek(FH, Len + sizeof(int), SEEK_CUR);
    } else Ret = PrintDosErr();
    --Rec;
  } while ((Rec) && (Ret));

  return Ret;
}


/**** Copy a string from Fortran to C ****/

void ForStrCpy(register char *Dest, register char *Src, ULONG Len)
{
  register ULONG  k;

  for(k = 0;(k < Len) && (Src[k] != ' ');++k)
    Dest[k] = Src[k];
}


/**** Writes a record in Fortran unformatted format ****/

BOOL ForWrite(FILE *FH, void *Num, ULONG Size)
{
  BOOL      Ret = TRUE;

#ifdef LITTLE_ENDIAN
  ULONG     Sz  = SwapI(Size);
#else
  ULONG     Sz  = Size;
#endif

  if ((fwrite(&Sz, sizeof(ULONG), 1, FH) != 1) ||
      (fwrite(Num, Size, 1, FH) != 1) ||
      (fwrite(&Sz, sizeof(ULONG), 1, FH) != 1))
    Ret = PrintDosErr();

  return Ret;
}


/**** Change a string to lowercase ****/

void Lower(register char *Str)
{
  do *Str = (char)tolower(*Str); while(*++Str);
}


/**** Free a memory list of atoms ****/

void PDBFree(register ATOMO *Atm)
{
  register ATOMO	*Succ;

  if (Atm) {
    do {
      Succ = Atm -> Ptr;
      SOFREE(Atm);
      Atm = Succ;
    } while(Atm);
  }
}


/**** Print an error with atom ****/

BOOL PrintAtmErr(char *Err, ATOMO *Atm)
{
  if (!GLOBSW_STDOUT)
    fprintf(stderr, "%s: %-7d %-4.4s %-2.2s %.4s_%.4s\n", Err,
            Atm -> Num, Atm -> Name.C, Atm -> Elem.C, Atm -> ResName.C,
            Atm -> ResSeq.C);

  return FALSE;
}


/**** Print loading format ****/

void PrintLoad(ULONG Msg, ULONG NMol)
{
  if ((!GLOBSW_STDOUT) && (NMol))
    printf(GetStr(MSG_UTILS_LOADPROG), NMol, GetStr(Msg));
}


/**** Print a progress message ****/

void PrintProg(ULONG Msg)
{
  if (!GLOBSW_STDOUT) printf("%s\n", GetStr(Msg));
}


/**** Remove a single atom from list ****/

void RemoveAtm(ATOMO **Atm, ATOMO **PrecAtm, ATOMO **InizAtm)
{
  if (*Atm == *InizAtm) {
    *InizAtm = (*Atm) -> Ptr;
    *PrecAtm = (*Atm) -> Ptr;
    FREE(*Atm);
    *Atm = *PrecAtm;
  } else {
    (*PrecAtm) -> Ptr = (*Atm) -> Ptr;
    if (((*Atm) -> Flags > 1) && ((*PrecAtm) -> Flags <= 1))
      (*PrecAtm) -> Flags += (BYTE)2;
    FREE(*Atm);
    *Atm = (*PrecAtm) -> Ptr;
  }
}


/**** Remove hydrogens ****/

void RemoveH(ATOMO **InizAtm, ULONG *TotAtm)
{
  register ATOMO   *Con;
  register UWORD   k, j;

  ATOMO            *PrecAtm = *InizAtm;
  ATOMO            *Atm     = *InizAtm;
  ULONG            Count    = 1;

  PrintProg(MSG_UTILS_RMHPROG);

  while(Atm) {
#ifdef LITTLE_ENDIAN
    if (Atm -> Elem.S == 0x48) {
#else
    if (Atm -> Elem.S == 0x4800) {
#endif
      for(Con = *InizAtm;Con;Con = Con -> Ptr) {
        for(k = 0;(k < MAXBOND) && (Con -> Conn[k]); ++k) {
          if (Con -> Conn[k] == Atm) {
            for(j = k; j < (MAXBOND - 1); ++j) {
              Con -> Conn[j]  = Con -> Conn[j + 1];
              Con -> Order[j] = Con -> Order[j + 1];
            }
          }
        }
      }
      --*TotAtm;
      RemoveAtm(&Atm, &PrecAtm, InizAtm);
    } else {
      PrecAtm    = Atm;
      Atm -> Num = Count++;
      Atm        = Atm -> Ptr;
    }
  }
}


/**** Removes water molecules ****/

void RemoveH2O(ATOMO **InizAtm, ULONG *TotAtm)
{
  ATOMO               *Atm     = *InizAtm;
  ATOMO               *PrecAtm = *InizAtm;
  register ULONG      Count    = 1;

  PrintProg(MSG_UTILS_RMWPROG);

  if (RenameWat(*InizAtm)) {
    Atm     = *InizAtm;
    PrecAtm = Atm;
    while(Atm) {
      if ((Atm -> ResName.C[0] ==  'H') && (Atm -> ResName.C[1] ==  'O') &&
          (Atm -> ResName.C[2] ==  'H')) {
        --*TotAtm;
        RemoveAtm(&Atm, &PrecAtm, InizAtm);
      } else {
        PrecAtm    = Atm;
        Atm -> Num = Count++;
        Atm        = Atm -> Ptr;
      }
    }
  }
}


/**** Remove a residue ****/

void RemoveRes(register ATOMO **Atm, ATOMO **PrecAtm, ATOMO **InizAtm,
               register ULONG *TotAtm)
{
  LONG          ResSeq = (*Atm) -> ResSeq.L;

  do {
    RemoveAtm(Atm, PrecAtm, InizAtm);
    --*TotAtm;
  } while((*Atm) && ((*Atm) -> ResSeq.L == ResSeq));
}


/**** Rename a residue ****/

ATOMO *RenameRes(register ATOMO *Atm, LONG Res)
{
  QCHAR             ResName;
  register ATOMO    *PrecAtm = Atm;
  register LONG     ResSeq   = Atm -> ResSeq.L;

  if ((Atm -> ResName.C[3] == '+') || (Atm -> ResName.C[3] == '-'))
    ((char *)&Res)[2] = Atm -> ResName.C[3];

  ResName.L = Atm -> ResName.L;
  while((Atm) && (ResName.L == Atm -> ResName.L) && (ResSeq == Atm -> ResSeq.L)) {
    Atm -> ResName.L = Res;
    PrecAtm = Atm;
    Atm     = Atm -> Ptr;
  }

  return PrecAtm;
}


/**** Rename waters ****/

BOOL RenameWat(register ATOMO *Atm)
{
  register ATOMO        *H1, *H2;

  BOOL			Ret  = FALSE;
  const char		*H2O = "HOH";

  while(Atm) {
    H1 = Atm -> Conn[0];
    H2 = Atm -> Conn[1];

#ifdef LITTLE_ENDIAN
    if ((Atm -> Elem.S == 0x4f) && (Atm -> NSost == 2) &&
        (H1 -> Elem.S == 0x48) && (H2 -> Elem.S == 0x48)) {
#else
    if ((Atm -> Elem.S == 0x4f00) && (Atm -> NSost == 2) &&
        (H1 -> Elem.S == 0x4800) && (H2 -> Elem.S == 0x4800)) {
#endif
      strncpy(Atm -> ResName.C, H2O, 3);
      strncpy(H1  -> ResName.C, H2O, 3);
      strncpy(H2  -> ResName.C, H2O, 3);
      Ret = TRUE;
    }
    Atm = Atm -> Ptr;
  }

  return Ret;
}


/**** Renumbers residues ****/

void RenRes(register ATOMO *Atm)
{
  char      Tmp[6];

  register ULONG  PrecResSeq = Atm -> ResSeq.L;
  register ULONG  ResSeq     = Prefs.RENSTART;

  PrintProg(MSG_UTILS_RENRESPROG);

  while(Atm) {
    if (Atm -> ResSeq.L != (LONG)PrecResSeq) {
      PrecResSeq = Atm -> ResSeq.L;
      ++ResSeq;
    }
    sprintf(Tmp, "%d", ResSeq);
    ResConv(&Atm -> ResSeq, Tmp);
    Atm = Atm -> Ptr;
  }
}


/**** Translates a string into ResSeq ****/

void ResConv(QCHAR *Res, register char *Str)
{
  register char   *Dest = (*Res).C;
  register UWORD  k;

  (*Res).L = (ULONG)NULL;
  for(k = 0;(*Str) && (k < 4); ++k) {
   *Dest = *Str;
   ++Str;
   ++Dest;
  }
}


/**** Set the residue type ****/

void SetTypeRes(QCHAR *Res, UBYTE Car)
{
  if (Res -> L != *(LONG *)"ASP-") {
    if ((Res -> C[3] == '+') || (Res -> C[3] == '-')) {
      Res -> C[2] = Res -> C[3];
    }
  } else Res -> L = *(LONG *)"AP-";

  Res -> C[3] = Car;
}


/**** Change the endian (if needed) and writes a long ****/

int SwapWrite(FILE *FH, void *Val)
{
  LONG          Num = *(LONG *)Val;
#ifdef LITTLE_ENDIAN
  Swap(&Num);
#endif

  return (int)fwrite(&Num, sizeof(ULONG), 1, FH);
}


/**** Change a string to uppercase ****/

void Upper(register char *Str)
{
  do *Str = (char)toupper(*Str); while(*++Str);
}


/**** Input prompt for a string ****/

void VegaGetString(char *Prompt, char *Str, UWORD Len)
{
  printf("%s: ",Prompt);
  fgets(Str, Len, stdin);
  Str[strlen(Str) - 1] = 0;
}

